var bridge = {
    init: false,
    isIOS: function() {
        const u = navigator.userAgent;
        if (u.match(/\(i[^;]+;( U;)? CPU.+Mac OS X/i)) {
            return true;
        } else {
            return false;
        }
    },
    isAndroid: function() {
        const u = navigator.userAgent;
        if (u.indexOf("Android") > -1 || u.indexOf("Linux") > -1) {
            return true;
        } else {
            return false;
        }
    },
    functions: {},
    callId: 0,
    callBackMap: new Map(),
    callNative: function(methodName, args, callback) {
        let callBackId = ++(this.callId);

        if (typeof methodName != "string") {
            console.error("param 'methodName' is not a string");
            return;
        }

        if (args != null && !(args instanceof Array)) {
            console.error("param 'args' is not a null or Array");
            return;
        }

        if (callback != null && typeof callback != "function") {
            console.error("param 'callback' is not a function");
            return;
        }

        if (callback) {
            this.callBackMap[callBackId] = callback;
        }

        let callInfo = {
            id: callBackId,
            methodName: methodName,
            args: args
        };

        if (this.isAndroid()) {
            _gdyNative.callNative(JSON.stringify(callInfo));
        }

        if (this.isIOS()) {
            window.webkit.messageHandlers.callNative.postMessage(JSON.stringify(callInfo));
        }

    },
    register: function(methodName, fun) {
        if (typeof fun != "function") {
            console.error("param 'fun' is not a function!!!");
            return;
        }
        let that = this;
        this.functions[methodName] = fun;
        if (!this.init) {
            this.init = true;
            setTimeout(function() {

                if (that.isAndroid()) {
                    _gdyNative.jsFunctionReady();
                }

                if (that.isIOS()) {
                    window.webkit.messageHandlers.jsFunctionReady.postMessage('');
                }

            }, 0);
        }
    },
    unRegister: function(methodName) {
        if (this.functions[methodName]) {
            delete this.functions[methodName];
        } else {
            console.log(`function ${methodName} is not exists`);
        }
    },
    /**
     *    callInfo: {
     *       method: "方法名",
     *       callBackId: 0,// 调用方法的id, 原封不动的传回去就行
     *       data: "['a','b', 2]" //调用方法时传入的参数
     *    }
     */
    handleMessageFromNative: function(callInfoString) {
        const callInfo = JSON.parse(callInfoString);
        let ret = {
            callBackId: callInfo.id,
            success: false
        };
        let fun = this.functions[callInfo.methodName];
        if (fun) {
            try {
                ret.data = fun.apply(null, callInfo.arg);
                if (typeof(ret.data) == "undefined") ret.data = null;
                ret.success = true;
            } catch (e) {
                ret.data = 'js function invoke catch error';
                console.error(e);
            }
        } else {
            ret.data = `js function ${callInfo.methodName} not exists`;
        }

        if (this.isAndroid()) {
            _gdyNative.returnValue(JSON.stringify(ret));
        }

        if (this.isIOS()) {
            window.webkit.messageHandlers.returnValue.postMessage(JSON.stringify(ret));
        }

    },
    /**
     *   resultInfo: {
     *       success: true|false, //调用是否成功
     *       callBackId: 0, //调用方法的id
     *       data: "" //stirng格式的数据, 可自行进行转换
     *   }
     *
     */
    postResult: function(resultInfoString) {
        const resultInfo = JSON.parse(resultInfoString);
        let callback = this.callBackMap[resultInfo.callBackId];
        if (callback) {
            this.callBackMap.delete(resultInfo.callBackId);
            callback(resultInfo.success, resultInfo.data);
        }
    }
};

! function() {
    if (navigator.userAgent.indexOf("GdyBridgeWebView") == -1) return;
    if (window._gdyBridge) return;
    window._gdyBridge = bridge;
}();